
package net.mcreator.enderitestuff.entity;

import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.network.PlayMessages;
import net.minecraftforge.network.NetworkHooks;
import net.minecraftforge.api.distmarker.OnlyIn;
import net.minecraftforge.api.distmarker.Dist;

import net.minecraft.world.level.Level;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.projectile.ItemSupplier;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.EntityType;
import net.minecraft.util.RandomSource;
import net.minecraft.sounds.SoundSource;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.network.protocol.game.ClientGamePacketListener;
import net.minecraft.network.protocol.Packet;

import net.mcreator.enderitestuff.init.EnderiteStuffModItems;
import net.mcreator.enderitestuff.init.EnderiteStuffModEntities;

@OnlyIn(value = Dist.CLIENT, _interface = ItemSupplier.class)
public class BeamEntity extends AbstractArrow implements ItemSupplier {
	public static final ItemStack PROJECTILE_ITEM = new ItemStack(EnderiteStuffModItems.POLISHED_OCANE.get());

	private int lifespan = 20;

	public BeamEntity(PlayMessages.SpawnEntity packet, Level world) {
		super(EnderiteStuffModEntities.BEAM.get(), world);
	}

	public BeamEntity(EntityType<? extends BeamEntity> type, Level world) {
		super(type, world);
	
	}

	public BeamEntity(EntityType<? extends BeamEntity> type, double x, double y, double z, Level world) {
		super(type, x, y, z, world);
	
	}

	public BeamEntity(EntityType<? extends BeamEntity> type, LivingEntity entity, Level world) {
		super(type, entity, world);
		
	}

	@Override
	public Packet<ClientGamePacketListener> getAddEntityPacket() {
		return NetworkHooks.getEntitySpawningPacket(this);
	}

	@Override
	@OnlyIn(Dist.CLIENT)
	public ItemStack getItem() {
		return PROJECTILE_ITEM;
	}

	@Override
	protected ItemStack getPickupItem() {
		return PROJECTILE_ITEM;
	}

	@Override
	protected void doPostHurtEffects(LivingEntity entity) {
		super.doPostHurtEffects(entity);
		entity.setArrowCount(entity.getArrowCount() - 1);
	}

	@Override
	public void tick() {
		super.tick();
		this.setNoGravity(true); // Ensures it stays floating

		if (this.inGround)
			this.discard();
		if (this.tickCount >= lifespan) {
        this.discard();
    }

		
	}
	@Override
	public boolean isNoGravity() {
    	return true; // Completely removes gravity
	}

	public static BeamEntity shoot(Level world, LivingEntity entity, RandomSource source) {
		return shoot(world, entity, source, 1f, 5, 5);
	}

	public static BeamEntity shoot(Level world, LivingEntity entity, RandomSource source, float pullingPower) {
		return shoot(world, entity, source, pullingPower * 1f, 5, 5);
	}

	public static BeamEntity shoot(Level world, LivingEntity entity, RandomSource random, float power, double damage, int knockback) {
		BeamEntity entityarrow = new BeamEntity(EnderiteStuffModEntities.BEAM.get(), entity, world);
		entityarrow.setDeltaMovement(entity.getViewVector(1).scale(power * 2));
		entityarrow.setSilent(true);
		entityarrow.setCritArrow(false);
		entityarrow.setBaseDamage(damage);
		entityarrow.setKnockback(knockback);
		world.addFreshEntity(entityarrow);
		world.playSound(null, entity.getX(), entity.getY(), entity.getZ(), ForgeRegistries.SOUND_EVENTS.getValue(new ResourceLocation("entity.guardian.attack")), SoundSource.PLAYERS, 1, 1f / (random.nextFloat() * 0.5f + 1) + (power / 2));
		return entityarrow;
	}
	

	public static BeamEntity shoot(LivingEntity entity, LivingEntity target) {
		BeamEntity entityarrow = new BeamEntity(EnderiteStuffModEntities.BEAM.get(), entity, entity.level());
		double dx = target.getX() - entity.getX();
		double dy = target.getY() + target.getEyeHeight() - 1.1;
		double dz = target.getZ() - entity.getZ();
		entityarrow.shoot(dx, dy - entityarrow.getY() + Math.hypot(dx, dz) * 0.2F, dz, 1f * 2, 12.0F);
		entityarrow.setSilent(true);
		entityarrow.setBaseDamage(5);
		entityarrow.setKnockback(5);
		entityarrow.setCritArrow(false);
		entity.level().addFreshEntity(entityarrow);
		entity.level().playSound(null, entity.getX(), entity.getY(), entity.getZ(), ForgeRegistries.SOUND_EVENTS.getValue(new ResourceLocation("entity.guardian.attack")), SoundSource.PLAYERS, 1, 1f / (RandomSource.create().nextFloat() * 0.5f + 1));
		return entityarrow;
	}
}
